//
// itractorgen - generate images of tractors
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <lib/config.h>
#include <cstdarg>
#include <cstdio>
#include <libexplain/output.h>

#include <itractorgen/expression.h>
#include <itractorgen/lex.h>


expression::~expression()
{
}


expression::expression() :
    pos(lex_get_position())
{
}


void
expression::fatal_error(const char *fmt, ...)
    const
{
    va_list ap;
    va_start(ap, fmt);
    char buf[1000];
    snprintf(buf, sizeof(buf), fmt, ap);
    va_end(ap);

    explain_output_error_and_die("%s: %s", pos.representation().c_str(), buf);
}


void
expression::lex_error(const char *fmt, ...)
    const
{
    va_list ap;
    va_start(ap, fmt);
    char buf[1000];
    snprintf(buf, sizeof(buf), fmt, ap);
    va_end(ap);

    explain_output_error_and_die("%s: %s", pos.representation().c_str(), buf);
    lex_one_more_error();
}


rectangle
expression::get_body_region(void)
    const
{
    return get_bounding_box();
}


// vim: set ts=8 sw=4 et :
