//
// itractorgen - generate images of tractors
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <itractorgen/expression/alternatives.h>
#include <itractorgen/expression/comma.h>
#include <itractorgen/expression/empty.h>


expression_alternatives::~expression_alternatives()
{
}


expression_alternatives::expression_alternatives(const choices_t &a_choices) :
    choices(a_choices)
{
}


expression_alternatives::pointer
expression_alternatives::create_func(const expression_comma *arguments)
{
    if (arguments->size() < 2)
    {
        arguments->lex_error
        (
            "function alternatives: requires at least 2 arguments, not %d",
            int(arguments->size())
        );
        return expression_empty::create();
    }

    choices_t pick_list;
    for (size_t j = 0; j < arguments->size(); ++j)
        pick_list.push_back(arguments->get(j));
    return pointer(new expression_alternatives(pick_list));
}


expression::pointer
expression_alternatives::resolve_alternatives(void)
    const
{
    if (choices.empty())
        return expression_empty::create();
    return choices[random() % choices.size()];
}


bool
expression_alternatives::is_an_image(void)
    const
{
    // needed for type checking function arguments
    assert(!choices.empty());
    return choices[0]->is_an_image();
}


bool
expression_alternatives::is_a_number(void)
    const
{
    // needed for type checking function arguments
    assert(!choices.empty());
    return choices[0]->is_a_number();
}


rectangle
expression_alternatives::get_bounding_box(void)
    const
{
    assert(!"supposed to call expression::resolve_alternatives before now");
    return rectangle();
}


rectangle
expression_alternatives::get_body_region(void)
    const
{
    assert(!"supposed to call expression::resolve_alternatives before now");
    return rectangle();
}


icon_pixel
expression_alternatives::get_pixel(double x, double y)
    const
{
    assert(!"supposed to call expression::resolve_alternatives before now");
    return choices[0]->get_pixel(x, y);
}


std::string
expression_alternatives::representation(void)
    const
{
    std::string result = "alternatives(";
    for
    (
        choices_t::const_iterator it = choices.begin();
        it != choices.end();
        ++it
    )
    {
        if (it != choices.begin())
            result += ", ";
        result += (*it)->representation();
    }
    result += ")";
    return result;
}


double
expression_alternatives::get_numeric_value(void)
    const
{
    assert(!"supposed to call expression::resolve_alternatives before now");
    return 0;
}




// vim: set ts=8 sw=4 et :
