//
// itractorgen - generate images of tractors
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef ITRACTORGEN_EXPRESSION_ALTERNATIVES_H
#define ITRACTORGEN_EXPRESSION_ALTERNATIVES_H

#include <lib/config.h>
#include <vector>

#include <itractorgen/expression.h>

class expression_comma; // forward

/**
  * The expression_alternatives class is used to represent an expression
  * tree that chooses amongst several alternatives, typically one of
  * several images, but not necessarily.
  */
class expression_alternatives:
    public expression
{
public:
    /**
      * The destructor.
      */
    virtual ~expression_alternatives();

    /**
      * The create class method is used to create new dynamically
      * allocated instances of this class.
      *
      * @param arguments
      *     The arguments to the function.  Don't worry, they are all
      *     checked before any are used.
      */
    static pointer create_func(const expression_comma *arguments);

protected:
    // See base class for documentation.
    bool is_an_image(void) const;

    // See base class for documentation.
    bool is_a_number(void) const;

    // See base class for documentation.
    rectangle get_bounding_box(void) const;

    // See base class for documentation.
    rectangle get_body_region(void) const;

    // See base class for documentation.
    icon_pixel get_pixel(double x, double y) const;

    // See base class for documentation.
    std::string representation(void) const;

    // See base class for documentation.
    double get_numeric_value(void) const;

    // See base class for documentation.
    pointer resolve_alternatives(void) const;

private:
    typedef std::vector<pointer> choices_t;

    /**
      * The constructor.
      * It is private on purpose, use the #create class method instead.
      *
      * @param choices
      *     The expression trees to choose from.
      */
    expression_alternatives(const choices_t &choices);

    /**
      * The choices instance variable is used to remember the expression
      * trees to choose from.
      */
    choices_t choices;

    /**
      * The default constructor.  Do not use.
      */
    expression_alternatives();

    /**
      * The copy constructor.  Do not use.
      *
      * @param rhs
      *     The right hand side of the initialization.
      */
    expression_alternatives(const expression_alternatives &rhs);

    /**
      * The assignment operator.  Do not use.
      *
      * @param rhs
      *     The right hand side of the assignment.
      */
    expression_alternatives &operator=(const expression_alternatives &rhs);
};

// vim: set ts=8 sw=4 et :
#endif // ITRACTORGEN_EXPRESSION_ALTERNATIVES_H
