//
// itractorgen - generate images of tractors
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <itractorgen/expression/body_region.h>
#include <itractorgen/expression/comma.h>
#include <itractorgen/expression/empty.h>


expression_body_region::~expression_body_region()
{
}


expression_body_region::expression_body_region(
    const expression::pointer &a_deeper,
    const rectangle &a_region
) :
    deeper(a_deeper),
    region(a_region)
{
}


expression::pointer
expression_body_region::create(const expression::pointer &a_deeper,
    const rectangle &a_region)
{
    return pointer(new expression_body_region(a_deeper, a_region));
}


expression::pointer
expression_body_region::create_func(const expression_comma *arguments)
{
    if (arguments->size() != 5)
    {
        arguments->lex_error
        (
            "body_region: requires 5 arguments, not %d",
            int(arguments->size())
        );
        return expression_empty::create();
    }

    expression::pointer a1 = arguments->get(0);
    if (!a1->is_an_image())
    {
        a1->lex_error("body_region: argument 1: not an image");
        return expression_empty::create();
    }

    rectangle r;
    expression::pointer a2 = arguments->get(1);
    if (!a2->is_a_number())
    {
        a2->lex_error("body_region: argument 2: not a number");
        return expression_empty::create();
    }
    r.x = a2->get_numeric_value();

    expression::pointer a3 = arguments->get(2);
    if (!a3->is_a_number())
    {
        a3->lex_error("body_region: argument 3: not a number");
        return expression_empty::create();
    }
    r.y = a3->get_numeric_value();

    expression::pointer a4 = arguments->get(3);
    if (!a4->is_a_number())
    {
        a4->lex_error("body_region: argument 4: not a number");
        return expression_empty::create();
    }
    r.dx = a4->get_numeric_value();

    expression::pointer a5 = arguments->get(4);
    if (!a5->is_a_number())
    {
        a5->lex_error("body_region: argument 5: not a number");
        return expression_empty::create();
    }
    r.dy = a5->get_numeric_value();

    //
    // Now we can call the regular create class method.
    //
    return create(a1, r);
}


rectangle
expression_body_region::get_bounding_box(void)
    const
{
    return deeper->get_bounding_box();
}


rectangle
expression_body_region::get_body_region(void)
    const
{
    return region;
}


double
expression_body_region::get_numeric_value(void)
    const
{
    assert(!"should not be able to reach here");
    return 0;
}


std::string
expression_body_region::representation(void)
    const
{
    return "body_region(" + deeper->representation() + ", "
        + region.representation() + ")";
}


icon_pixel
expression_body_region::get_pixel(double x, double y)
    const
{
    return deeper->get_pixel(x, y);
}


bool
expression_body_region::is_an_image(void)
    const
{
    return true;
}


bool
expression_body_region::is_a_number(void)
    const
{
    return false;
}


expression::pointer
expression_body_region::resolve_alternatives(void)
    const
{
    return create(deeper->resolve_alternatives(), region);
}


// vim: set ts=8 sw=4 et :
