//
// itractorgen - generate images of tractors
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <itractorgen/expression/comma.h>


expression_comma::~expression_comma()
{
}


expression_comma::expression_comma()
{
}


expression_comma::pointer
expression_comma::create(void)
{
    return pointer(new expression_comma());
}


void
expression_comma::push_back(const expression::pointer &arg)
{
    params.push_back(arg);
}


size_t
expression_comma::size(void)
    const
{
    return params.size();
}


expression::pointer
expression_comma::get(size_t n)
    const
{
    assert(n < params.size());
    return params[n];
}


std::string
expression_comma::representation(void)
    const
{
    std::string result;
    for (size_t j = 0; j < size(); ++j)
    {
        if (j)
            result += ", ";
        result += get(j)->representation();
    }
    return result;
}


bool
expression_comma::is_a_number(void)
    const
{
    return false;
}


double
expression_comma::get_numeric_value(void)
    const
{
    assert(!"should not be able to get to here");
    return 0;
}


bool
expression_comma::is_an_image(void)
    const
{
    return false;
}


icon_pixel
expression_comma::get_pixel(double, double)
    const
{
    assert(!"should not be able to get to here");
    return icon_pixel();
}


rectangle
expression_comma::get_bounding_box(void)
    const
{
    assert(!"should not be able to get to here");
    return rectangle();
}


rectangle
expression_comma::get_body_region(void)
    const
{
    assert(!"should not be able to get to here");
    return rectangle();
}


expression::pointer
expression_comma::resolve_alternatives(void)
    const
{
    assert(!"should not be able to get to here");
    return create();
}


// vim: set ts=8 sw=4 et :
