//
// itractorgen - generate images of tractors
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef ITRACTORGEN_EXPRESSION_COMMA_H
#define ITRACTORGEN_EXPRESSION_COMMA_H

#include <lib/config.h>
#include <vector>

#include <itractorgen/expression.h>

/**
  * The expression_comma class is used to represent a comma-separated
  * argument list, usually passed to a function.
  */
class expression_comma:
    public expression
{
public:
    typedef boost::shared_ptr<expression_comma> pointer;

    /**
      * The destructor.
      */
    virtual ~expression_comma();

    /**
      * The create class method is used to create new dynamically
      * allocated instances of this class.
      */
    static pointer create(void);

    void push_back(const expression::pointer &arg);

    size_t size(void) const;

    expression::pointer get(size_t n) const;

protected:
    // See base class for documentation.
    bool is_an_image(void) const;

    // See base class for documentation.
    bool is_a_number(void) const;

    // See base class for documentation.
    rectangle get_bounding_box(void) const;

    // See base class for documentation.
    rectangle get_body_region(void) const;

    // See base class for documentation.
    icon_pixel get_pixel(double x, double y) const;

    // See base class for documentation.
    std::string representation(void) const;

    // See base class for documentation.
    double get_numeric_value(void) const;

    // See base class for documentation.
    expression::pointer resolve_alternatives(void) const;

private:
    typedef std::vector<expression::pointer> params_t;

    params_t params;

    /**
      * The default constructor.
      * It is private on purpose, use the #create class method instead.
      */
    expression_comma();

    /**
      * The copy constructor.  Do not use.
      *
      * @param rhs
      *     The right hand side of the initialization.
      */
    expression_comma(const expression_comma &rhs);

    /**
      * The assignment operator.  Do not use.
      *
      * @param rhs
      *     The right hand side of the assignment.
      */
    expression_comma &operator=(const expression_comma &rhs);
};

// vim: set ts=8 sw=4 et :
#endif // ITRACTORGEN_EXPRESSION_COMMA_H
