//
// itractorgen - generate images of tractors
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <itractorgen/expression/crop.h>


expression_crop::~expression_crop()
{
}


expression_crop::expression_crop(
    const pointer &a_deeper,
    const rectangle &a_bb
) :
    deeper(a_deeper),
    bb(a_bb)
{
}


expression_crop::pointer
expression_crop::create(const pointer &a_deeper, const rectangle &a_bb)
{
    return pointer(new expression_crop(a_deeper, a_bb));
}


bool
expression_crop::is_an_image(void)
    const
{
    return true;
}


bool
expression_crop::is_a_number(void)
    const
{
    return false;
}


rectangle
expression_crop::get_bounding_box(void)
    const
{
    return bb;
}


rectangle
expression_crop::get_body_region(void)
    const
{
    return bb * deeper->get_body_region();
}


icon_pixel
expression_crop::get_pixel(double x, double y)
    const
{
    if (!bb.contains(x, y))
        return icon_pixel::transparent;
    return deeper->get_pixel(x, y);
}


std::string
expression_crop::representation(void)
    const
{
    std::string result = "crop(";
    result += deeper->representation();
    result += ", ";
    result += bb.representation();
    result += ")";
    return result;
}


double
expression_crop::get_numeric_value(void)
    const
{
    return 0;
}


expression::pointer
expression_crop::resolve_alternatives(void)
    const
{
    return create(deeper, bb);
}


// vim: set ts=8 sw=4 et :
