//
// itractorgen - generate images of tractors
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef ITRACTORGEN_EXPRESSION_CROP_H
#define ITRACTORGEN_EXPRESSION_CROP_H

#include <itractorgen/expression.h>

/**
  * The expression_crop class is used to represent the processing
  * required to crop an image to a specific boundary.
  */
class expression_crop:
    public expression
{
public:
    /**
      * The destructor.
      */
    virtual ~expression_crop();

    /**
      * The create class method is used to create new dynamically
      * allocated instances of this class.
      *
      * @param deeper
      *     The image to be cropped.
      * @param bb
      *     The bounding box to use.
      */
    static pointer create(const pointer &deeper, const rectangle &bb);

private:
    /**
      * The constructor.
      * It is private on purpose, use the #create class method instead.
      *
      * @param deeper
      *     The image to be cropped.
      * @param bb
      *     The bounding box to use.
      */
    expression_crop(const pointer &deeper, const rectangle &bb);

protected:
    // See base class for documentation.
    bool is_an_image(void) const;

    // See base class for documentation.
    bool is_a_number(void) const;

    // See base class for documentation.
    rectangle get_bounding_box(void) const;

    // See base class for documentation.
    rectangle get_body_region(void) const;

    // See base class for documentation.
    icon_pixel get_pixel(double x, double y) const;

    // See base class for documentation.
    std::string representation(void) const;

    // See base class for documentation.
    double get_numeric_value(void) const;

    // See base class for documentation.
    pointer resolve_alternatives(void) const;

private:
    /**
      * The deeper instance variable is used to remember the image to be
      * cropped.
      */
    pointer deeper;

    /**
      * The bb instance variable is used to remember the bounding box to
      * use.
      */
    rectangle bb;

    /**
      * The default constructor.  Do not use.
      */
    expression_crop();

    /**
      * The copy constructor.  Do not use.
      *
      * @param rhs
      *     The right hand side of the initialization.
      */
    expression_crop(const expression_crop &rhs);

    /**
      * The assignment operator.  Do not use.
      *
      * @param rhs
      *     The right hand side of the assignment.
      */
    expression_crop &operator=(const expression_crop &rhs);
};

// vim: set ts=8 sw=4 et :
#endif // ITRACTORGEN_EXPRESSION_CROP_H
