//
// itractorgen - generate images of tractors
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <itractorgen/expression/image.h>


expression_image::~expression_image()
{
}


expression_image::expression_image(const std::string &a_filename) :
    filename(a_filename)
{
}


expression_image::pointer
expression_image::create(const std::string &a_filename)
{
    return pointer(new expression_image(a_filename));
}


bool
expression_image::is_an_image(void)
    const
{
    return true;
}


rectangle
expression_image::get_bounding_box(void)
    const
{
    if (!img)
        img = image::create_from_file(filename);
    return rectangle(0, 0, img->get_width(), img->get_height());
}


rectangle
expression_image::get_body_region(void)
    const
{
    return get_bounding_box();
}


icon_pixel
expression_image::get_pixel(double x, double y)
    const
{
    if (!img)
        img = image::create_from_file(filename);
    return img->get_pixel(floor(x), floor(y));
}


std::string 
expression_image::representation(void)
    const
{
    return filename;
}


bool
expression_image::is_a_number(void)
    const
{
    return false;
}


double
expression_image::get_numeric_value(void)
    const
{
    assert(!"should not be able to reach here");
    return 0;
}


expression_image::expression_image(
    const std::string &a_filename,
    const image::pointer &a_img
) :
    filename(a_filename),
    img(a_img)
{
}


expression::pointer
expression_image::resolve_alternatives(void)
    const
{
    return pointer(new expression_image(filename, img));
}


// vim: set ts=8 sw=4 et :
