//
// itractorgen - generate images of tractors
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef ITRACTORGEN_EXPRESSION_IMAGE_H
#define ITRACTORGEN_EXPRESSION_IMAGE_H

#include <lib/config.h>
#include <string>

#include <lib/image.h>
#include <itractorgen/expression.h>

/**
  * The expression_image class is used to represent an expression that
  * is an image to be drawn onto the output.
  */
class expression_image:
    public expression
{
public:
    /**
      * The destructor.
      */
    virtual ~expression_image();

    /**
      * The create class method is used to create new dynamically
      * allocated instances of this class.
      *
      * @param filename
      *     The name of the file containoing the image to draw into the output.
      */
    static pointer create(const std::string &filename);

protected:
    // See base class for documentation.
    bool is_an_image(void) const;

    // See base class for documentation.
    rectangle get_bounding_box(void) const;

    // See base class for documentation.
    rectangle get_body_region(void) const;

    // See base class for documentation.
    icon_pixel get_pixel(double x, double y) const;

    // See base class for documentation.
    std::string representation(void) const;

    // See base class for documentation.
    bool is_a_number(void) const;

    // See base class for documentation.
    double get_numeric_value(void) const;

    // See base class for documentation.
    pointer resolve_alternatives(void) const;

private:
    /**
      * The constructor.
      * It is private on purpose, use the #create class method instead.
      *
      * @param filename
      *     The name of the file containoing the image to draw into the output.
      */
    expression_image(const std::string &filename);

    /**
      * The constructor.
      * It is private on purpose, use the #create class method instead.
      *
      * @param filename
      *     The name of the file containoing the image to draw into the output.
      * @param img
      *     the image data being accessed.
      */
    expression_image(const std::string &filename, const image::pointer &img);

    /**
      * The filename instance variable is used to remember the name of
      * the file containing the image to be draw onto the output.
      */
    std::string filename;

    /**
      * The img instance variable is used to remember the image data
      * being accessed.  It will be NULL until the file actually needs
      * to be accessed.
      */
    mutable image::pointer img;

    /**
      * The default constructor.  Do not use.
      */
    expression_image();

    /**
      * The copy constructor.  Do not use.
      *
      * @param rhs
      *     The right hand side of the initialization.
      */
    expression_image(const expression_image &rhs);

    /**
      * The assignment operator.  Do not use.
      *
      * @param rhs
      *     The right hand side of the assignment.
      */
    expression_image &operator=(const expression_image &rhs);
};

// vim: set ts=8 sw=4 et :
#endif // ITRACTORGEN_EXPRESSION_IMAGE_H
