//
// itractorgen - generate images of tractors
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef ITRACTORGEN_EXPRESSION_NUMBER_H
#define ITRACTORGEN_EXPRESSION_NUMBER_H

#include <itractorgen/expression.h>

/**
  * The expression_number class is used to represent an expression that
  * is a numeric value.
  */
class expression_number:
    public expression
{
public:
    /**
      * The destructor.
      */
    virtual ~expression_number();

    /** 
      * The create class method is used to create new dynamically
      * allocated instances of this class.
      *
      * @param value
      *     The value of the expression.
      */ 
    static pointer create(double value);

protected:
    // See base class for documentation.
    bool is_an_image(void) const;

    // See base class for documentation.
    bool is_a_number(void) const;

    // See base class for documentation.
    rectangle get_bounding_box(void) const;

    // See base class for documentation.
    rectangle get_body_region(void) const;

    // See base class for documentation.
    icon_pixel get_pixel(double x, double y) const;

    // See base class for documentation.
    std::string representation(void) const;

    // See base class for documentation.
    double get_numeric_value(void) const;

    // See base class for documentation.
    pointer resolve_alternatives(void) const;

private:
    /**
      * The constructor.
      * It is private on purpose, use the #create class method instead.
      *
      * @param value
      *     The value of the expression.
      */
    expression_number(double value);

    /**
      * The value instance variable is used to remember the value of the
      * expression.
      */
    double value;

    /**
      * The default constructor.  Do not use.
      */
    expression_number();

    /**
      * The copy constructor.  Do not use.
      *
      * @param rhs
      *     The right hand side of the initialization.
      */
    expression_number(const expression_number &rhs);

    /**
      * The assignment operator.  Do not use.
      *
      * @param rhs
      *     The right hand side of the assignment.
      */
    expression_number &operator=(const expression_number &rhs);
};

// vim: set ts=8 sw=4 et :
#endif // ITRACTORGEN_EXPRESSION_NUMBER_H
