//
// itractorgen - generate images of tractors
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef ITRACTORGEN_EXPRESSION_OVER_H
#define ITRACTORGEN_EXPRESSION_OVER_H

#include <itractorgen/expression.h>

class expression_comma; // forward

/**
  * The expression_over class is used to represent the operation of
  * compositing one image over the top of another.
  */
class expression_over:
    public expression
{
public:
    /**
      * The destructor.
      */
    virtual ~expression_over();

    /** 
      * The create class method is used to create new dynamically
      * allocated instances of this class.
      *
      * @param upper
      *     The image on the top.
      * @param lower
      *     The image on the bottom.
      */ 
    static pointer create(const expression::pointer &upper,
        const expression::pointer &lower);

    /** 
      * The create_func class method is used to create new dynamically
      * allocated instances of this class, based on functions arguments
      * seen in the grammar.
      *
      * @param arguments
      *     The arguments to the function.  Don't worry, they are all
      *     checked before any are used.
      */ 
    static pointer create_func(const expression_comma *arguments);

protected:
    // See base class for documentation.
    bool is_an_image(void) const;

    // See base class for documentation.
    bool is_a_number(void) const;

    // See base class for documentation.
    rectangle get_bounding_box(void) const;

    // See base class for documentation.
    rectangle get_body_region(void) const;

    // See base class for documentation.
    icon_pixel get_pixel(double x, double y) const;

    // See base class for documentation.
    std::string representation(void) const;

    // See base class for documentation.
    double get_numeric_value(void) const;

    // See base class for documentation.
    pointer resolve_alternatives(void) const;

private:
    /**
      * The constructor.
      * It is private on purpose, use the #create class method instead.
      *
      * @param upper
      *     The image on the top.
      * @param lower
      *     The image on the bottom.
      */
    expression_over(const expression::pointer &upper,
        const expression::pointer &lower);

    /**
      * The upper instance variable is used to remember the image on the
      * top.
      */
    expression::pointer upper;

    /**
      * The lower instance variable is used to remember the image on the
      * bottom.
      */
    expression::pointer lower;

    /**
      * The default constructor.  Do not use.
      */
    expression_over();

    /**
      * The copy constructor.  Do not use.
      *
      * @param rhs
      *     The right hand side of the initialization.
      */
    expression_over(const expression_over &rhs);

    /**
      * The assignment operator.  Do not use.
      *
      * @param rhs
      *     The right hand side of the assignment.
      */
    expression_over &operator=(const expression_over &rhs);
};

// vim: set ts=8 sw=4 et :
#endif // ITRACTORGEN_EXPRESSION_OVER_H
