//
// itractorgen - generate images of tractors
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef ITRACTORGEN_EXPRESSION_SCALE_H
#define ITRACTORGEN_EXPRESSION_SCALE_H

#include <itractorgen/expression.h>

class expression_comma; // forward

/**
  * The expression_scale class is used to represent an expression operation
  * that scales an image.
  */
class expression_scale:
    public expression
{
public:
    /**
      * The destructor.
      */
    virtual ~expression_scale();

    /**
      * The create class method is used to create new dynamically
      * allocated instances of this class.
      *
      * @param deeper
      *     The image to be scaled.
      * @param sx
      *     The amount to scale the X coordinate
      * @param sy
      *     The amount to scale the Y coordinate
      */
    static pointer create(const pointer &deeper, double sx, double sy);

    /**
      * The create_func class method is used to create new dynamically
      * allocated instances of this class.
      *
      * @param arguments
      *     The arguments to the function.  Don't worry, they are all
      *     checked before any are used.
      */
    static pointer create_func(const expression_comma *arguments);

protected:
    // See base class for documentation.
    bool is_an_image(void) const;

    // See base class for documentation.
    rectangle get_bounding_box(void) const;

    // See base class for documentation.
    rectangle get_body_region(void) const;

    // See base class for documentation.
    icon_pixel get_pixel(double x, double y) const;

    // See base class for documentation.
    bool is_a_number(void) const;

    // See base class for documentation.
    double get_numeric_value(void) const;

    // See base class for documentation.
    std::string representation(void) const;

    // See base class for documentation.
    pointer resolve_alternatives(void) const;

private:
    /**
      * The constructor.  It is private on purpose, use a "create" class
      * method instead.
      *
      * @param deeper
      *     The image to be scaled.
      * @param sx
      *     The amount to scale the X coordinate
      * @param sy
      *     The amount to scale the Y coordinate
      */
    expression_scale(const pointer &deeper, double sx, double sy);

    /**
      * The deeper instance variable is used to remember the image to be
      * scaled.
      */
    pointer deeper;

    /**
      * The sx instance variable is used to remember the amount to scale
      * the X coordinate
      */
    double sx;

    /**
      * The sy instance variable is used to remember the amount to scale
      * the Y coordinate
      */
    double sy;

    /**
      * The isx instance variable is used to remember the inverse of the
      * amount to scale the X coordinate
      */
    double isx;

    /**
      * The isy instance variable is used to remember the inverse of the
      * amount to scale the Y coordinate
      */
    double isy;

    /**
      * The default constructor.  Do not use.
      */
    expression_scale();

    /**
      * The copy constructor.  Do not use.
      */
    expression_scale(const expression_scale &);

    /**
      * The assignment operator.  Do not use.
      */
    expression_scale &operator=(const expression_scale &);
};

// vim: set ts=8 sw=4 et :
#endif // ITRACTORGEN_EXPRESSION_SCALE_H
