//
// itractorgen - generate images of tractors
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <itractorgen/expression.h>


icon_pixel
expression::super_sample(int x, int y, unsigned grid)
    const
{
    if (grid < 2)
        grid = 2;
    double incr = 1.0 / grid;
    double begn = incr / 2.0;
    icon_pixel::real_type r_sum = 0;
    icon_pixel::real_type g_sum = 0;
    icon_pixel::real_type b_sum = 0;
    icon_pixel::real_type a_sum = 0;
    unsigned n = 0;
    for (double dy = begn; dy < 1; dy += incr)
    {
        double py = y + dy;
        for (double dx = begn; dx < 1; dx += incr)
        {
            double px = x + dx;
            icon_pixel p = get_pixel(px, py);
            double r = p.get_red();
            double g = p.get_green();
            double b = p.get_blue();
            double a = p.get_alpha();
            if (a <= 0)
            {
#if 0
                r = 1;
                g = 1;
                b = 1;
#else
                continue;
#endif
            }
            r_sum += r;
            g_sum += g;
            b_sum += b;
            a_sum += a;
            ++n;
        }
    }
    if (n == 0)
        return icon_pixel::transparent;
    double inv = 1. / n;
    icon_pixel result(r_sum * inv, g_sum * inv, b_sum * inv, a_sum * inv);
    return result;
}


// vim: set ts=8 sw=4 et :
