//
// itractorgen - generate images of tractors
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <lib/config.h>
#include <cassert>

#include <lib/string_from_format.h>

#include <itractorgen/position.h>


position::~position()
{
}


position::position(const std::string &a_file_name, int a_line_number) :
    file_name(a_file_name),
    line_number(a_line_number)
{
    assert(line_number >= 1);
}


position::position(const position &rhs) :
    file_name(rhs.file_name),
    line_number(rhs.line_number)
{
}


position &
position::operator=(const position &rhs)
{
    if (this != &rhs)
    {
        file_name = rhs.file_name;
        line_number = rhs.line_number;
    }
    return *this;
}


std::string
position::representation(void)
    const
{
    return file_name + string_from_format(": %d", line_number);
}


// vim: set ts=8 sw=4 et :
