//
// itractorgen - generate images of tractors
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef ITRACTORGEN_POSITION_H
#define ITRACTORGEN_POSITION_H

#include <lib/config.h>
#include <string>

/**
  * The position class is used to represent a position within a source
  * file, for generating error messages.
  */
class position
{
public:
    /**
      * The destructor.
      * It is not virtual, thou shalt not derive from this class.
      */
    ~position();

    /**
      * The constructor.
      *
      * @param file_name
      *     The name of the file in question.
      * @param line_number
      *     The number of the text line within the file (counting from one).
      */
    position(const std::string &file_name, int line_number);

    /**
      * The representation method may be used to obtain a string that
      * includes bioth the file name and the line number.
      */
    std::string representation(void) const;

    /**
      * The copy constructor.
      *
      * @param rhs
      *     The right hand side of the initialization.
      */
    position(const position &rhs);

    /**
      * The assignment operator.
      *
      * @param rhs
      *     The right hand side of the assignment.
      */
    position &operator=(const position &rhs);

private:
    /**
      * The file_name instance variable is used to remember the name of
      * the file in question.
      */
    std::string file_name;

    /**
      * The line_number instance variable is used to remember the number
      * of the text line within the file (counting from one).
      */
    int line_number;

    /**
      * The default constructor.  Do not use.
      */
    position();
};

// vim: set ts=8 sw=4 et :
#endif // ITRACTORGEN_POSITION_H
