//
// itractorgen - generate images of tractors
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <lib/config.h>
#include <cassert>
#include <cerrno>
#include <libexplain/fopen.h>
#include <libexplain/output.h>

#include <lib/dirstack.h>
#include <lib/dirstack/functor.h>
#include <lib/path_join.h>


dirstack::~dirstack()
{
}


dirstack::dirstack()
{
}


dirstack::dirstack(const dirstack &rhs) :
    directories(rhs.directories)
{
}


dirstack &
dirstack::operator=(const dirstack &rhs)
{
    if (this != &rhs)
        directories = rhs.directories;
    return *this;
}


void
dirstack::push_back(const std::string &path)
{
    directories.push_back(path);
}


void
dirstack::push_front(const std::string &path)
{
    directories.push_front(path);
}


FILE *
dirstack::fopen(const std::string &filename, const char *flags)
{
    assert(!directories.empty());
    for
    (
        directories_t::const_iterator it = directories.begin();
        it != directories.end();
        ++it
    )
    {
        std::string directory = *it;
        std::string path = path_join(directory, filename);
        FILE *fp = ::fopen(path.c_str(), flags);
        if (fp)
            return fp;
        if (errno != ENOENT)
        {
            explain_output_error_and_die
            (
                "%s",
                explain_fopen(path.c_str(), flags)
            );
        }
    }
    std::string path = path_join(directories.front(), filename);
    return explain_fopen_or_die(path.c_str(), flags);
}


std::string
dirstack::find(const std::string &filename)
{
    assert(!directories.empty());
    for
    (
        directories_t::const_iterator it = directories.begin();
        it != directories.end();
        ++it
    )
    {
        std::string directory = *it;
        std::string path = path_join(directory, filename);
        if (access(path.c_str(), F_OK) == 0)
            return path;
    }
    return std::string();
}


void
dirstack::for_each(dirstack_functor &func)
    const
{
    for
    (
        directories_t::const_iterator it = directories.begin();
        it != directories.end();
        ++it
    )
    {
        std::string directory = *it;
        if (!func(directory))
            break;
    }
}


// vim: set ts=8 sw=4 et :
