//
// itractorgen - generate images of tractors
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIB_DIRSTACK_FUNCTOR_H
#define LIB_DIRSTACK_FUNCTOR_H

#include <lib/config.h>
#include <string>

/**
  * The dirstack_functor class is used to represent a process to be
  * applied to a directory stack (see the #dirstack class).
  */
class dirstack_functor
{
public:
    /**
      * The destructor.
      */
    virtual ~dirstack_functor();

protected:
    /**
      * The default constructor.
      * For use by derived classes only.
      */
    dirstack_functor();

public:
    /**
      * The function call operator is used to process each directory ina
      * directory stack.  It is called like a function, that's why it's
      * called a functor.
      *
      * @param directory
      *     The directory to process.
      */
    bool
    operator()(const std::string &directory)
    {
        return walk(directory);
    }

private:
    /**
      * The walk method is used to process each directory in a directory stack.
      *
      * @param directory
      *     The directory to process.
      */
    virtual bool walk(const std::string &directory) = 0;

private:
    /**
      * The copy constructor.  Do not use.
      *
      * @param rhs
      *     The right hand side of the initialization.
      */
    dirstack_functor(const dirstack_functor &rhs);

    /**
      * The assignment operator.  Do not use.
      *
      * @param rhs
      *     The right hand side of the assignment.
      */
    dirstack_functor &operator=(const dirstack_functor &rhs);
};

// vim: set ts=8 sw=4 et :
#endif // LIB_DIRSTACK_FUNCTOR_H
