//
// itractorgen - generate images of tractors
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <lib/config.h>
#include <algorithm>
#include <cassert>
#include <cstdio>
#include <cstdlib>

#include <lib/dirstack/functor/list.h>
#include <lib/ends_with.h>
#include <lib/read_whole_directory.h>


dirstack_functor_list::~dirstack_functor_list()
{
}


dirstack_functor_list::dirstack_functor_list()
{
}


bool
dirstack_functor_list::walk(const std::string &path)
{
    names_t partial;
    read_whole_directory(path, partial);
    for
    (
        names_t::const_iterator it = partial.begin();
        it != partial.end();
        ++it
    )
    {
        std::string name = *it;
        if (!ends_with(name, ".tractor"))
            continue;
        name = name.substr(0, name.size() - 8);
        if (std::find(names.begin(), names.end(), name) == names.end())
            names.push_back(name);
    }

    return true;
}


void
dirstack_functor_list::list(void)
    const
{
    names_t final = names;
    final.sort();
    for
    (
        names_t::const_iterator it = final.begin();
        it != final.end();
        ++it
    )
    {
        std::string name = *it;
        printf("%s\n", name.c_str());
    }
}


std::string
dirstack_functor_list::random(void)
    const
{
    if (names.empty())
        return std::string();
    size_t n = ::random() % names.size();
    for (names_t::const_iterator it = names.begin(); ; ++it, --n)
    {
        assert(it != names.end());
        if (!n)
            return *it;
    }
}


// vim: set ts=8 sw=4 et :
