//
// itractorgen - generate images of tractors
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIB_DIRSTACK_FUNCTOR_LIST_H
#define LIB_DIRSTACK_FUNCTOR_LIST_H

#include <lib/config.h>
#include <list>

#include <lib/dirstack/functor.h>

/**
  * The dirstack_functor_list class is used to represent the processing
  * required to list the contents of the directory stack.
  */
class dirstack_functor_list:
    public dirstack_functor
{
public:
    /**
      * The destructor.
      */
    virtual ~dirstack_functor_list();

    /**
      * The default constructor.
      */
    dirstack_functor_list();

    /**
      * The list method is sued to print each of the discovered style
      * names on the standard output, sorted, one per line.
      * If the list is empty, nothing will be printed.
      */
    void list(void) const;

    /**
      * The random method may be used to select one of the discovered
      * style names, at random.  If the list is empty, the empty string
      * will be returned.
      */
    std::string random(void) const;

private:
    // See base class for documentation.
    bool walk(const std::string &directory);

    typedef std::list<std::string> names_t;

    /**
      * The names instance variable is used to remember the discovered
      * list of style names.  If a file of the same name appears in more
      * than one of the directories in the search path, it will only be
      * present in the list once.
      */
    names_t names;

    /**
      * The copy constructor.  Do not use.
      *
      * @param rhs
      *     The right hand side of the initialization.
      */
    dirstack_functor_list(const dirstack_functor_list &rhs);

    /**
      * The assignment operator.  Do not use.
      *
      * @param rhs
      *     The right hand side of the assignment.
      */
    dirstack_functor_list &operator=(const dirstack_functor_list &rhs);
};

// vim: set ts=8 sw=4 et :
#endif // LIB_DIRSTACK_FUNCTOR_LIST_H
