//
// itractorgen - generate images of tractors
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <lib/image.h>


void
image::copy(const image::pointer &src)
{
    copy(src, 0, 0);
}


static inline int
min(int a, int b)
{
    return (a < b ? a : b);
}


static inline int
max(int a, int b)
{
    return (a > b ? a : b);
}


void
image::copy(const image::pointer &src, int ox, int oy)
{
    int x_min = max(0, ox);
    int y_min = max(0, oy);
    int x_max = min(ox + src->get_width(), get_width());
    int y_max = min(oy + src->get_height(), get_height());

    for (int y = y_min; y < y_max; ++y)
    {
        for (int x = x_min; x < x_max; ++x)
        {
            icon_pixel value;
            src->get_pixel(x - ox, y - oy, value);
            set_pixel(x, y, value);
        }
    }
}


// vim: set ts=8 sw=4 et :
