//
// itractorgen - generate images of tractors
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <lib/image/crop.h>


image_crop::~image_crop()
{
}


image::pointer
image_crop::create(image::pointer a_deeper, int a_ox, int a_oy, int a_dx,
    int a_dy)
{
    return pointer(new image_crop(a_deeper, a_ox, a_oy, a_dx, a_dy));
}


image_crop::image_crop(
    image::pointer a_deeper,
    int a_ox,
    int a_oy,
    int a_dx,
    int a_dy
) :
    deeper(a_deeper),
    ox(a_ox),
    oy(a_oy),
    dx(a_dx),
    dy(a_dy)
{
    if (dx < 0)
        dx = 1;
    if (dy < 0)
        dy = 1;
}


unsigned
image_crop::get_width(void)
    const
{
    return dx;
}


unsigned
image_crop::get_height(void)
    const
{
    return dy;
}


void
image_crop::get_pixel(int x, int y, icon_pixel &value)
    const
{
    if (x < 0 || x >= dx || y < 0 || y >= dy)
    {
        value = icon_pixel::transparent;
        return;
    }
    deeper->get_pixel(x + ox, y + oy, value);
}


void
image_crop::set_pixel(int x, int y, const icon_pixel &value)
{
    if (x >= 0 && x < dx && y >= 0 && y < dy)
        deeper->set_pixel(x + ox, y + oy, value);
}


const char *
image_crop::type_name(void)
    const
{
    return deeper->type_name();
}


void
image_crop::save_to_file(const std::string &filename)
    const
{
    deeper->save_to_file(filename);
}


// vim: set ts=8 sw=4 et :
