//
// itractorgen - generate images of tractors
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIB_IMAGE_CROP_H
#define LIB_IMAGE_CROP_H

#include <lib/image.h>

/**
  * The image_crop class is used to represent a filter which crops
  * another image, making it appear to be a different size than it is.
  */
class image_crop:
    public image
{
public:
    /**
      * The destructor.
      */
    virtual ~image_crop();

    /**
      * The create class method is used to create a new instance of the
      * crop filter, from the given image.
      *
      * @param deeper
      *     The image to be filtered.
      * @param ox
      *     The X offset into the image to be filtered.
      * @param oy
      *     The Y offset into the image to be filtered.
      * @param dx
      *     The width of the chunk of the image to be filtered.
      * @param dy
      *     The height of the chunk of the image to be filtered.
      */
    static image::pointer create(image::pointer deeper, int ox, int oy, int dx,
        int dy);

protected:
    // See base class for documentation.
    unsigned get_width(void) const;

    // See base class for documentation.
    unsigned get_height(void) const;

    // See base class for documentation.
    void get_pixel(int x, int y, icon_pixel &value);

    // See base class for documentation.
    void set_pixel(int x, int y, const icon_pixel &value);

    // See base class for documentation.
    const char *type_name(void) const;

    // See base class for documentation.
    void get_pixel(int x, int y, icon_pixel &value) const;

    // See base class for documentation.
    void save_to_file(const std::string &filename) const;

private:
    image::pointer deeper;
    int ox;
    int oy;
    int dx;
    int dy;

    /**
      * The default constructor.  Do not use.
      */
    image_crop();

    /**
      * The constructor.
      */
    image_crop(image::pointer deeper, int ox, int oy, int dx, int dy);

    /**
      * The copy constructor.  Do not use.
      */
    image_crop(const image_crop &);

    /**
      * The assignment operator.  Do not use.
      */
    image_crop &operator=(const image_crop &);
};

// vim: set ts=8 sw=4 et :
#endif // LIB_IMAGE_CROP_H
