//
// itractorgen - generate images of tractors
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIB_IMAGE_FORMAT_H
#define LIB_IMAGE_FORMAT_H

#include <lib/image.h>

/**
  * The image_format class is used to represent an image of a specific
  * format, e.g. GIF or PNG.
  */
class image_format:
    public image
{
public:
    /**
      * The destructor.
      */
    virtual ~image_format();

protected:
    // See base class for documentation.
    void get_pixel(int x, int y, icon_pixel &value) const;

    // See base class for documentation.
    void set_pixel(int x, int y, const icon_pixel &value);

    // See base class for documentation.
    unsigned get_width(void) const;

    // See base class for documentation.
    unsigned get_height(void) const;

    typedef unsigned char data_t;

    const data_t *get_data(void) const;

    const data_t *
    get_data(int y)
        const
    {
        return (data + 4 * width * y);
    }

    const data_t *
    get_data(int x, int y)
        const
    {
        return (data + 4 * (width * y + x));
    }

    /**
      * The save_to_file method is used to store the image data into a
      * file on disk.
      *
      * @param filename
      *     The name of the file to be written.
      */
    virtual void save_to_file(const std::string &filename) const;

    /**
      * The constructor.
      *
      * @param width
      *     The width of the image
      * @param height
      *     The height of the image
      * @param data
      *     The data is ordered as row then column then component (RGBA).
      */
    image_format(int width, int height, data_t *data);

    /**
      * The constructor.
      *
      * The image will be completely blank and transparent.
      *
      * @param width
      *     The width of the image
      * @param height
      *     The height of the image
      */
    image_format(int width, int height);

private:
    unsigned width;
    unsigned height;
    data_t *data;

    /**
      * The default constructor.  Do not use.
      */
    image_format();

    /**
      * The copy constructor.  Do not use.
      */
    image_format(const image_format &);

    /**
      * The assignment operator.  Do not use.
      */
    image_format &operator=(const image_format &);
};

// vim: set ts=8 sw=4 et :
#endif // LIB_IMAGE_FORMAT_H
