//
// itractorgen - generate images of tractors
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIB_IMAGE_FORMAT_GIF_H
#define LIB_IMAGE_FORMAT_GIF_H

#include <lib/image/format.h>

/**
  * The image_format_gif class is used to represent an image to be
  * loaded from a GIF file.
  */
class image_format_gif:
    public image_format
{
public:
    /**
      * The destructor.
      */
    virtual ~image_format_gif();

    /**
      * The create class method is used to create an image instance from
      * a GIF file on disk.
      *
      * @param filename
      *     The name of the file to be read
      */
    static image::pointer create(const std::string &filename);

protected:
    // See base class for documentation.
    const char *type_name(void) const;

private:
    /**
      * The default constructor.  Do not use.
      */
    image_format_gif();

    /**
      * The constructor.
      */
    image_format_gif(int width, int height, data_t *data);

    /**
      * The copy constructor.  Do not use.
      */
    image_format_gif(const image_format_gif &);

    /**
      * The assignment operator.  Do not use.
      */
    image_format_gif &operator=(const image_format_gif &);
};

// vim: set ts=8 sw=4 et :
#endif // LIB_IMAGE_FORMAT_GIF_H
