//
// itractorgen - generate images of tractors
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <lib/image.h>


void
image::overlay_pixel(int x, int y)
{
    overlay_pixel(x, y, pen_clr);
}


void
image::overlay_pixel(int x, int y, const icon_pixel &p1)
{
    if (p1.is_transparent())
        return;
    if (p1.is_opaque())
    {
        set_pixel(x, y, p1);
    }
    else
    {
        icon_pixel p0;
        get_pixel(x, y, p0);
        if (p0.is_transparent())
        {
            set_pixel(x, y, p1);
        }
        else
        {
            icon_pixel::real_type p1a = p1.get_alpha();
            icon_pixel::real_type aa = 1. - p1a;
            icon_pixel::real_type r =
                p0.get_red() * aa + p1.get_red() * p1a;
            icon_pixel::real_type g =
                p0.get_green() * aa + p1.get_green() * p1a;
            icon_pixel::real_type b =
                p0.get_blue() * aa + p1.get_blue() * p1a;
            icon_pixel::real_type a =
                p0.get_alpha() * aa + p1a;
            set_pixel(x, y, icon_pixel(r, g, b, a));
        }
    }
}


// vim: set ts=8 sw=4 et :
