//
// itractorgen - generate images of tractors
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <cstring>

#include <lib/libdir.h>


static bool
replace(std::string &text, const std::string &lhs, const std::string &rhs)
{
    const char *p = strstr(text.c_str(), lhs.c_str());
    if (!p)
        return false;
    size_t n = p - text.c_str();
    text =
        (
            text.substr (0, n)
        +
            rhs
        +
            text.substr(n + lhs.size(), text.size() - (n + lhs.size()))
        );
    return true;
}


static std::string
fix(const std::string &text)
{
    std::string result = text;
    for (;;)
    {
        if (replace(result, "${prefix}", "/usr/local"))
            continue;
        if (replace(result, "${exec_prefix}", "/usr/local"))
            continue;
        if (replace(result, "${datarootdir}", "/usr/local/share"))
            continue;
        return result;
    }
}


std::string
configured_bindir(void)
{
    static std::string cached;
    if (cached.empty())
        cached = fix("/usr/local/bin");
    return cached;
}


std::string
configured_datarootdir(void)
{
    static std::string cached;
    if (cached.empty())
        cached = fix("/usr/local/share");
    return cached;
}


std::string
configured_datadir(void)
{
    static std::string cached;
    if (cached.empty())
        cached = fix("/usr/local/share");
    return cached;
}


std::string
configured_libdir(void)
{
    static std::string cached;
    if (cached.empty())
        cached = fix("/usr/local/lib");
    return cached;
}


std::string
configured_localstatedir(void)
{
    static std::string cached;
    if (cached.empty())
        cached = fix("${prefix}/var");
    return cached;
}


std::string
configured_sysconfdir(void)
{
    static std::string cached;
    if (cached.empty())
        cached = fix("${prefix}/etc");
    return cached;
}


// vim: set ts=8 sw=4 et :
