//
// itractorgen - generate images of tractors
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIB_LIBDIR_H
#define LIB_LIBDIR_H

#include <lib/config.h>
#include <string>

/**
  * The configured_bindir function is used to obtain the configured
  * value for the diectory containing executable files.  It defaults
  * to ${prefix}/bin.  This is manipulated by the ./configure --bindir
  * option.
  */
std::string configured_bindir(void);

/**
  * The configured_datadir function is used to obtain the
  * configured value for shared data.  It defaults to
  * ${prefix}/share/icon-compositor.  This is manipulated by the
  * ./configure --datadir option.
  */
std::string configured_datadir(void);
std::string configured_datarootdir(void);

/**
  * The configured_libdir function is used to obtain the configured
  * value for architecture specific data.  It defaults to
  * ${prefix}/lib/icon-compositor.  This is manipulated by the
  * ./configure --libdir option.
  */
std::string configured_libdir(void);

/**
  * The configured_localstatedir function is used to obtain
  * the configured value for varying data.  It defaults to
  * ${prefix}/var/icon-compositor.  This is manipulated by the
  * ./configure --localstatedir option.
  */
std::string configured_localstatedir(void);

/**
  * The configured_sysconfdir function is used to obtain the configured
  * value for system configuration data.  It defaults to ${prefix}/etc.
  * This is manipulated by the ./configure --sysconfdir option.
  */
std::string configured_sysconfdir(void);

// vim: set ts=8 sw=4 et :
#endif // LIB_LIBDIR_H
