//
// itractorgen - generate images of tractors
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIB_PROGRESS_H
#define LIB_PROGRESS_H

/**
  * The progress class is used to represent and ascii progress bar which
  * can be printed on a terminal (stdout).
  */
class progress
{
public:
    /**
      * The destructor.
      */
    virtual ~progress();

    /**
      * The constructor.
      */
    progress(int min, int max);

    void begin();
    void middle(int pos);
    void end();
    void clear();

    /**
      * The set_show method may be used to override the default show
      * flag (which is based on tty output).
      *
      * @param yesno
      *     true if show, false if don't
      */
    void set_show(bool yesno);

private:
    bool show;
    int width;
    bool need_to_clear;
    int minimum;
    int maximum;
    int digits;
    double start_time;
    double next;

    /**
      * The default constructor.  Do not use.
      */
    progress();

    /**
      * The copy constructor.  Do not use.
      */
    progress(const progress &);

    /**
      * The assignment operator.  Do not use.
      */
    progress &operator=(const progress &);
};

// vim: set ts=8 sw=4 et :
#endif // LIB_PROGRESS_H
