//
// itractorgen - generate images of tractors
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <lib/config.h>
#include <cstdlib>
#include <cstring>
#include <libexplain/program_name.h>

#include <lib/libdir.h>
#include <lib/search_path.h>


dirstack search_path;


void
search_path_init(void)
{
    //
    // The itractorgen_search_path environment variable is considered
    // mostly to facilitate automatic testing.  Aegis will set
    // this environment variable before running tests (see the
    // aegis.conf.d/test file for where this is configured).
    //
    std::string envar_name(explain_program_name_get());
    envar_name += "_search_path";
    const char *cp = getenv(envar_name.c_str());
    if (cp && *cp)
    {
        for (;;)
        {
            const char *colon = strpbrk(cp, ":;");
            if (!colon)
            {
                if (*cp)
                    search_path.push_back(cp);
                break;
            }
            if (cp != colon)
                search_path.push_back(std::string(cp, colon - cp));
            cp = colon + 1;
        }
    }

    //
    // The system shared directory is the last place we will look.
    //
    search_path.push_back
    (
        configured_datadir() + "/" + explain_program_name_get()
    );
}


// vim: set ts=8 sw=4 et :
