#!/bin/sh
#
# itractorgen - generate images of tractors
# Copyright (C) 2011 Peter Miller
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
TEST_SUBJECT="expression_body_region"
. test_prelude

cat > left.pbm << 'fubar'
P1
8 8
0000000001000000010000000100000001000000010000000111111000000000
fubar
test $? -eq 0 || no_result

cat > middle.pbm << 'fubar'
P1
8 8
0000000001000100011011000101010001000100010001000100011000000000
fubar
test $? -eq 0 || no_result

cat > right.pbm << 'fubar'
P1
8 8
0000000001111000010001000100010001111000010001000100001000000000
fubar
test $? -eq 0 || no_result

cat > test.script << 'fubar'
left = body_region(left.pbm, 0, 0, 7, 8);
middle = body_region(middle.pbm, 0, 0, 7, 8);
right = body_region(right.pbm, 0, 0, 7, 8);
fubar
test $? -eq 0 || no_result

cat > test.ok << 'fubar'
#define test_out_width 29
#define test_out_height 8
static char test_out_bits[] = {
  0x00, 0x00, 0x00, 0x00, 0x02, 0x91, 0xC8, 0x03, 0x02, 0x9B, 0x4D, 0x04, 
  0x02, 0x95, 0x4A, 0x04, 0x02, 0x91, 0xC8, 0x03, 0x02, 0x91, 0x48, 0x04, 
  0x7E, 0xB1, 0x58, 0x08, 0x00, 0x00, 0x00, 0x00, };
fubar
test $? -eq 0 || no_result

# This is a test of the script parsing.
itractorgen -n 6 -f test.script -o test_out.png
if test $? -ne 0 ; then fail; fi

# we have to convert the output to an ascii format
convert test_out.png test_out.xbm
test $? -eq 0 || no_result

# make sure we get what we expected
diff test.ok test_out.xbm
if test $? -ne 0 ; then fail; fi

#
# Only definite negatives are possible.
# The functionality exercised by this test appears to work,
# no other guarantees are made.
#
pass
# vim: set ts=8 sw=4 et :
